@@ -196,7 +196,9 @@ get_deltas (gpointer handle)
 	char *comp_str;
 	char *time_string = NULL;
 	char t_str [100]; 
+	const char *serv_time;
 	struct stat buf;
+	static GStaticMutex connecting = G_STATIC_MUTEX_INIT;
         
 	if (!handle)
 		return FALSE;
@@ -210,20 +212,25 @@ get_deltas (gpointer handle)
 	if (priv->mode == CAL_MODE_LOCAL)
 		return FALSE;
 
-	g_strlcpy (t_str, e_cal_backend_cache_get_server_utc_time (cache), 100);
-	if (!*t_str || !strcmp (t_str, "")) {
-		icaltimetype temp;
-		time_t current_time;
-		const struct tm *tm;
+	g_static_mutex_lock (&connecting);
 
-		g_warning (" Could not get the correct time stamp for using in getQuick Messages\n");
-		temp = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
-		current_time = icaltime_as_timet_with_zone (temp, icaltimezone_get_utc_timezone ());
-		tm = gmtime (&current_time);
-		strftime (t_str, 100, "%Y-%m-%dT%H:%M:%SZ", tm);
+	serv_time = e_cal_backend_cache_get_server_utc_time (cache);
+	if (serv_time) {
+		g_strlcpy (t_str, e_cal_backend_cache_get_server_utc_time (cache), 100);
+		if (!*t_str || !strcmp (t_str, "")) {
+			icaltimetype temp;
+			time_t current_time;
+			const struct tm *tm;
+
+			g_warning (" Could not get the correct time stamp for using in getQuick Messages\n");
+			temp = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
+			current_time = icaltime_as_timet_with_zone (temp, icaltimezone_get_utc_timezone ());
+			tm = gmtime (&current_time);
+			strftime (t_str, 100, "%Y-%m-%dT%H:%M:%SZ", tm);
+		}
+		time_string = g_strdup (t_str);
 	}
 
-	time_string = g_strdup (t_str);
 	status = e_gw_connection_get_quick_messages (cnc, cbgw->priv->container_id, "attachments recipients message recipientStatus default peek", &time_string, "New", "CalendarItem", NULL,  -1,  &item_list);
 	
 	if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
@@ -231,10 +238,13 @@ get_deltas (gpointer handle)
 	
 	if (status != E_GW_CONNECTION_STATUS_OK) {
 				
-		if (status == E_GW_CONNECTION_STATUS_NO_RESPONSE) 
+		if (status == E_GW_CONNECTION_STATUS_NO_RESPONSE) {
+			g_static_mutex_unlock (&connecting);
 			return TRUE;
+		}
 
 		e_cal_backend_groupwise_notify_error_code (cbgw, status);
+		g_static_mutex_unlock (&connecting);
 		return TRUE;
 	}
 	/* store the timestamp in the cache */	
@@ -282,10 +292,13 @@ get_deltas (gpointer handle)
 		
 	g_free (time_string);
 	if (status != E_GW_CONNECTION_STATUS_OK) {
-		if (status == E_GW_CONNECTION_STATUS_NO_RESPONSE) 
+		if (status == E_GW_CONNECTION_STATUS_NO_RESPONSE) { 
+			g_static_mutex_unlock (&connecting);
 			return TRUE;
+		}
 
 		e_cal_backend_groupwise_notify_error_code (cbgw, status);
+		g_static_mutex_unlock (&connecting);
 		return TRUE;
 	}
 
@@ -328,10 +341,13 @@ get_deltas (gpointer handle)
 		status = e_gw_connection_get_quick_messages (cnc, cbgw->priv->container_id, "iCalId", NULL, "All", "CalendarItem", NULL,  -1,  &item_list);
 
 	if (status != E_GW_CONNECTION_STATUS_OK) {
-		if (status == E_GW_CONNECTION_STATUS_NO_RESPONSE) 
+		if (status == E_GW_CONNECTION_STATUS_NO_RESPONSE) {
+			g_static_mutex_unlock (&connecting);
 			return TRUE;
+		}
 
 		e_cal_backend_groupwise_notify_error_code (cbgw, status);
+		g_static_mutex_unlock (&connecting);
 		return TRUE;
 	}
 
@@ -378,10 +394,31 @@ get_deltas (gpointer handle)
 		g_slist_free (cache_keys);
 		item_list = NULL;
 	}
+	
+	g_static_mutex_unlock (&connecting);
 		
         return TRUE;        
 }
 
+static gboolean
+get_deltas_timeout (gpointer cbgw)
+{
+	GThread *thread;
+	GError *error = NULL;
+
+	if (!cbgw)
+		return FALSE;
+
+	thread = g_thread_create ((GThreadFunc) get_deltas, cbgw, FALSE, &error);
+	if (!thread) {
+		g_warning (G_STRLOC ": %s", error->message);
+		g_error_free (error);
+	}
+
+	return TRUE;
+}
+
+
 static char* 
 form_uri (ESource *source)
 {
@@ -468,7 +505,7 @@ cache_init (ECalBackendGroupwise *cbgw)
 
 			/*  Set up deltas only if it is a Calendar backend */
 			if (kind == ICAL_VEVENT_COMPONENT)
-				priv->timeout_id = g_timeout_add (time_interval, (GSourceFunc) get_deltas, (gpointer) cbgw);
+				priv->timeout_id = g_timeout_add (time_interval, (GSourceFunc) get_deltas_timeout, (gpointer) cbgw);
 			priv->mode = CAL_MODE_REMOTE;
 			return GNOME_Evolution_Calendar_Success;
 		}
@@ -495,7 +532,7 @@ cache_init (ECalBackendGroupwise *cbgw)
 		/* get the deltas from the cache */
 		if (get_deltas (cbgw)) {
 			if (kind == ICAL_VEVENT_COMPONENT)
-				priv->timeout_id = g_timeout_add (time_interval, (GSourceFunc) get_deltas, (gpointer) cbgw);
+				priv->timeout_id = g_timeout_add (time_interval, (GSourceFunc) get_deltas_timeout, (gpointer) cbgw);
 			priv->mode = CAL_MODE_REMOTE;
 			return GNOME_Evolution_Calendar_Success;
 		} else {
@@ -546,7 +583,7 @@ connect_to_server (ECalBackendGroupwise *cbgw)
 		/* we need to read actual rights from server when we implement proxy user access */
 		cbgw->priv->read_only = FALSE;
 
-		if (priv->cnc && priv->cache) {
+		if (priv->cnc && priv->cache && priv->container_id) {
 			priv->mode = CAL_MODE_REMOTE;
 			if (priv->mode_changed && !priv->timeout_id && (e_cal_backend_get_kind (E_CAL_BACKEND (cbgw)) == ICAL_VEVENT_COMPONENT)) {
 				GThread *thread1;
@@ -560,7 +597,7 @@ connect_to_server (ECalBackendGroupwise *cbgw)
 					e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Could not create thread for getting deltas"));
 					return GNOME_Evolution_Calendar_OtherError;
 				}
-				priv->timeout_id = g_timeout_add (CACHE_REFRESH_INTERVAL, (GSourceFunc) get_deltas, (gpointer) cbgw);
+				priv->timeout_id = g_timeout_add (CACHE_REFRESH_INTERVAL, (GSourceFunc) get_deltas_timeout, (gpointer)cbgw);
 
 			}
 	
@@ -764,6 +801,10 @@ e_cal_backend_groupwise_get_static_capabilities (ECalBackendSync *backend, EData
 				  CAL_STATIC_CAPABILITY_NO_CONV_TO_ASSIGN_TASK "," \
 				  CAL_STATIC_CAPABILITY_NO_CONV_TO_RECUR "," \
 				  CAL_STATIC_CAPABILITY_REQ_SEND_OPTIONS "," \
+				  CAL_STATIC_CAPABILITY_ORGANIZER_MUST_ACCEPT "," \
+				  CAL_STATIC_CAPABILITY_DELEGATE_SUPPORTED "," \
+				  CAL_STATIC_CAPABILITY_DELEGATE_TO_MANY "," \
+				  CAL_STATIC_CAPABILITY_NO_ORGANIZER "," \
 				  CAL_STATIC_CAPABILITY_SAVE_SCHEDULES);
 
 	return GNOME_Evolution_Calendar_Success;
@@ -1135,6 +1176,7 @@ e_cal_backend_groupwise_get_object_list (ECalBackendSync *backend, EDataCal *cal
 		}
         }
 
+	g_object_unref (cbsexp);
 	g_list_foreach (components, (GFunc) g_object_unref, NULL);
 	g_list_free (components);
 
@@ -1211,7 +1253,7 @@ typedef struct {
 } ECalBackendGroupwiseComputeChangesData;
 
 static void
-e_cal_backend_groupwise_compute_changes_foreach_key (const char *key, gpointer data)
+e_cal_backend_groupwise_compute_changes_foreach_key (const char *key, const char *value, gpointer data)
 {
 	ECalBackendGroupwiseComputeChangesData *be_data = data;
                 
@@ -1497,6 +1539,7 @@ e_cal_backend_groupwise_modify_object (ECalBackendSync *backend, EDataCal *cal,
 	ECalComponent *comp, *cache_comp = NULL;
 	EGwConnectionStatus status;
 	EGwItem *item, *cache_item;
+	const char *uid = NULL;
 
 	*old_object = NULL;
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
@@ -1517,19 +1560,39 @@ e_cal_backend_groupwise_modify_object (ECalBackendSync *backend, EDataCal *cal,
 
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, icalcomp);
-	item = e_gw_item_new_from_cal_component (priv->container_id, cbgw, comp);
+	e_cal_component_get_uid (comp, &uid);
 
 	/* check if the object exists */
 	switch (priv->mode) {
 	case CAL_MODE_ANY :
 	case CAL_MODE_REMOTE :
 		/* when online, send the item to the server */
-		cache_comp = e_cal_backend_cache_get_component (priv->cache, e_gw_item_get_icalid (item), NULL);
+		cache_comp = e_cal_backend_cache_get_component (priv->cache, uid, NULL);
 		if (!cache_comp) {
 			g_message ("CRITICAL : Could not find the object in cache");
 			return GNOME_Evolution_Calendar_ObjectNotFound;
 		}
 
+		if (e_cal_backend_groupwise_utils_check_delegate (comp, e_gw_connection_get_user_email (priv->cnc))) {
+			item = e_gw_item_new_for_delegate_from_cal (cbgw, comp);
+			status = e_gw_connection_delegate_request (priv->cnc, item, e_gw_item_get_id (item), NULL, NULL, NULL); 
+		
+			if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
+					status = e_gw_connection_delegate_request (priv->cnc, item, e_gw_item_get_id (item), NULL, NULL, NULL);
+				
+				if (status != E_GW_CONNECTION_STATUS_OK) {
+					g_object_unref (comp);
+					g_object_unref (cache_comp);
+					return GNOME_Evolution_Calendar_OtherError;
+				}
+			
+			
+			e_cal_backend_cache_put_component (priv->cache, comp);
+			*new_object = e_cal_component_get_as_string (comp);
+			break;
+		}
+
+		item = e_gw_item_new_from_cal_component (priv->container_id, cbgw, comp);
 		cache_item =  e_gw_item_new_from_cal_component (priv->container_id, cbgw, cache_comp);
 		if ( e_gw_item_get_item_type (item) == E_GW_ITEM_TYPE_TASK) {
 			gboolean completed, cache_completed;
@@ -1552,6 +1615,7 @@ e_cal_backend_groupwise_modify_object (ECalBackendSync *backend, EDataCal *cal,
 				break;
 			}
 		}
+		
 		e_gw_item_set_changes (item, cache_item); 
 
 		/* the second argument is redundant */
